@@ -27,6 +27,7 @@ import re
 import string
 import unicodedata
 import gettext
+import traceback
 from gi import require_version
 require_version('IBus', '1.0')
 from gi.repository import IBus
@@ -51,6 +52,18 @@ DOMAINNAME = 'ibus-typing-booster'
 _ = lambda a: gettext.dgettext(DOMAINNAME, a)
 N_ = lambda a: a
 
+# When matching keybindings, the following modifiers
+# for all bits set in this mask are ignored:
+KEYBINDING_IGNORE_MASK = (
+    IBus.ModifierType.LOCK_MASK | # Caps Lock
+    IBus.ModifierType.MOD2_MASK   # Num Lock
+)
+
+# The number of current imes needs to be limited to some fixed
+# maximum number because of the property menu to select the preëdit
+# ime. Unfortunately the number of sub-properties for such a menu
+# cannot be changed, as a workaround a fixed number can be used
+# and unused entries can be hidden.
 MAXIMUM_NUMBER_OF_INPUT_METHODS = 10
 
 NORMALIZATION_FORM_INTERNAL = 'NFD'
@@ -1678,6 +1691,43 @@ def tokenize(text):
         tokens.append(strip_token(word))
     return tokens
 
+def color_string_to_argb(color_string):
+    '''
+    Converts a color string to a 32bit  ARGB value
+
+    :param color_string: The color to convert to 32bit ARGB
+    :type color_string: String
+                        - Standard name from the X11 rgb.txt
+                        - Hex value: “#rgb”, “#rrggbb”, “#rrrgggbbb”
+                                     or ”#rrrrggggbbbb”
+                        - RGB color: “rgb(r,g,b)”
+                        - RGBA color: “rgba(r,g,b,a)”
+    :rtype: Integer
+
+    Examples:
+
+    >>> print('%x' %color_string_to_argb('rgb(0xff, 0x10, 0x25)'))
+    ffff1025
+
+    >>> print('%x' %color_string_to_argb('#108040'))
+    ff108040
+
+    >>> print('%x' %color_string_to_argb('#fff000888'))
+    ffff0088
+
+    >>> print('%x' %color_string_to_argb('#ffff00008888'))
+    ffff0088
+
+    >>> print('%x' %color_string_to_argb('rgba(0xff, 0x10, 0x25, 0.5)'))
+    7fff1025
+    '''
+    gdk_rgba = Gdk.RGBA()
+    gdk_rgba.parse(color_string)
+    return (((int(gdk_rgba.alpha * 0xff) & 0xff) << 24)
+            + ((int(gdk_rgba.red * 0xff) & 0xff) << 16)
+            + ((int(gdk_rgba.green * 0xff) & 0xff) << 8)
+            + ((int(gdk_rgba.blue * 0xff) & 0xff)))
+
 def is_ascii(text):
     '''Checks whether all characters in text are ASCII characters
 
@@ -1815,8 +1865,7 @@ def bidi_embed(text):
     '''
     if is_right_to_left(text):
         return chr(0x202B) + text + chr(0x202C) # RLE + text + PDF
-    else:
-        return chr(0x202A) + text + chr(0x202C) # LRE + text + PDF
+    return chr(0x202A) + text + chr(0x202C) # LRE + text + PDF
 
 def contains_letter(text):
     '''Returns whether “text” contains a “letter” type character
@@ -1849,16 +1898,15 @@ def variant_to_value(variant):
     type_string = variant.get_type_string()
     if type_string == 's':
         return variant.get_string()
-    elif type_string == 'i':
+    if type_string == 'i':
         return variant.get_int32()
-    elif type_string == 'b':
+    if type_string == 'b':
         return variant.get_boolean()
-    elif type_string == 'v':
+    if type_string == 'v':
         return variant.unpack()
-    elif len(type_string) > 0 and type_string[0] == 'a':
+    if len(type_string) > 0 and type_string[0] == 'a':
         return variant.unpack()
-    else:
-        print('error: unknown variant type: %s' %type_string)
+    print('error: unknown variant type: %s' %type_string)
     return variant
 
 def dict_update_existing_keys(pdict, other_pdict):
@@ -2061,13 +2109,13 @@ def get_ime_help(ime_name):
     :param ime_name: Name of the input method
     :type ime_name: String
     :rtype: tuple of the form (path, title, description, full_contents, error)
-            where “path” is the full path of the file implementing the input method,
-            “title” is title of the input method,
+            where “path” is the full path of the file implementing the
+            input method, “title” is title of the input method,
             “description” is the description of the input method,
             and full_contents is the full content of the file implementing
             the input method.
-            “error” is empty if getting help for the input method was successfull,
-            otherwise it might contain an error message.
+            “error” is empty if getting help for the input method was
+            successfull, otherwise it might contain an error message.
 
     “NoIme” is handled as a special case.
 
@@ -2118,7 +2166,10 @@ def get_ime_help(ime_name):
         return ('', '', '', '', ('m17n dir not found, tried: %s' %dirnames))
     path = os.path.join(m17n_dir, mim_file)
     try:
-        with open(path, mode='r', encoding='UTF-8', errors='ignore') as ime_file:
+        with open(path,
+                  mode='r',
+                  encoding='UTF-8',
+                  errors='ignore') as ime_file:
             full_contents = ime_file.read()
     except FileNotFoundError:
         return ('', '', '', '', _('File not found'))
@@ -2151,17 +2202,16 @@ def xdg_save_data_path(*resource):
     '''
     if IMPORT_XDG_BASEDIRECTORY_SUCCESSFUL:
         return xdg.BaseDirectory.save_data_path(*resource)
-    else:
-        # Replicate implementation of xdg.BaseDirectory.save_data_path
-        # here:
-        xdg_data_home = os.environ.get('XDG_DATA_HOME') or os.path.join(
-            os.path.expanduser('~'), '.local', 'share')
-        resource = os.path.join(*resource)
-        assert not resource.startswith('/')
-        path = os.path.join(xdg_data_home, resource)
-        if not os.path.isdir(path):
-            os.makedirs(path)
-        return path
+    # Replicate implementation of xdg.BaseDirectory.save_data_path
+    # here:
+    xdg_data_home = os.environ.get('XDG_DATA_HOME') or os.path.join(
+        os.path.expanduser('~'), '.local', 'share')
+    resource = os.path.join(*resource)
+    assert not resource.startswith('/')
+    path = os.path.join(xdg_data_home, resource)
+    if not os.path.isdir(path):
+        os.makedirs(path)
+    return path
 
 class KeyEvent:
     '''Key event class used to make the checking of details of the key
@@ -2244,7 +2294,7 @@ class KeyEvent:
                self.release))
 
 def keyevent_to_keybinding(keyevent):
-    keybinding=''
+    keybinding = ''
     if keyevent.shift:
         keybinding += 'Shift+'
     if keyevent.lock:
@@ -2307,18 +2357,22 @@ class HotKeys:
         for command in keybindings:
             for keybinding in keybindings[command]:
                 key = keybinding_to_keyevent(keybinding)
+                val = key.val
+                state = key.state & ~KEYBINDING_IGNORE_MASK
                 if command in self._hotkeys:
-                    self._hotkeys[command].append((key.val, key.state))
+                    self._hotkeys[command].append((val, state))
                 else:
-                    self._hotkeys[command] = [(key.val, key.state)]
+                    self._hotkeys[command] = [(val, state)]
 
     def __contains__(self, command_key_tuple):
         if not isinstance(command_key_tuple, tuple):
             return False
         command = command_key_tuple[1]
         key = command_key_tuple[0]
+        val = key.val
+        state = key.state & ~KEYBINDING_IGNORE_MASK
         if command in self._hotkeys:
-            if (key.val, key.state) in self._hotkeys[command]:
+            if (val, state) in self._hotkeys[command]:
                 return True
         return False
 
@@ -2328,6 +2382,9 @@ class HotKeys:
 class ItbKeyInputDialog(Gtk.MessageDialog):
     def __init__(
             self,
+            # Translators: This is used in the title bar of a dialog window
+            # requesting that the user types a key to be used as a new
+            # key binding for a command.
             title=_('Key input'),
             parent=None):
         Gtk.Dialog.__init__(
@@ -2336,9 +2393,16 @@ class ItbKeyInputDialog(Gtk.MessageDialog):
             parent=parent)
         self.add_button(_('Cancel'), Gtk.ResponseType.CANCEL)
         self.set_modal(True)
-        self.set_markup('<big><b>%s</b></big>'
-                        % _('Please press a key (or a key combination)'))
+        self.set_markup(
+            '<big><b>%s</b></big>'
+            # Translators: This is from the dialog to enter a key or a
+            # key combination to be used as a key binding for a
+            # command.
+            % _('Please press a key (or a key combination)'))
         self.format_secondary_text(
+            # Translators: This is from the dialog to enter a key or a
+            # key combination to be used as a key binding for a
+            # command.
             _('The dialog will be closed when the key is released'))
         self.connect('key_press_event', self.on_key_press_event)
         self.connect('key_release_event', self.on_key_release_event)
@@ -2348,10 +2412,11 @@ class ItbKeyInputDialog(Gtk.MessageDialog):
 
     def on_key_press_event(# pylint: disable=no-self-use
             self, widget, event):
-        widget.e = (event.keyval, event.get_state())
+        widget.e = (event.keyval,
+                    event.get_state() & ~KEYBINDING_IGNORE_MASK)
         return True
     def on_key_release_event(# pylint: disable=no-self-use
-            self, widget, dummy_event):
+            self, widget, _event):
         widget.response(Gtk.ResponseType.OK)
         return True
 
@@ -2411,14 +2476,14 @@ class ItbAboutDialog(Gtk.AboutDialog):
         self.show()
 
     def on_close_aboutdialog( # pylint: disable=no-self-use
-            self, dummy_about_dialog, dummy_response):
+            self, _about_dialog, _response):
         '''
         The “About” dialog has been closed by the user
 
-        :param dummy_about_dialog: The “About” dialog
-        :type dummy_about_dialog: GtkDialog object
-        :param dummy_response: The response when the “About” dialog was closed
-        :type dummy_response: Gtk.ResponseType enum
+        :param _about_dialog: The “About” dialog
+        :type _about_dialog: GtkDialog object
+        :param _response: The response when the “About” dialog was closed
+        :type _response: Gtk.ResponseType enum
         '''
         self.destroy()
 
